@@ -127,7 +127,7 @@ static void msg_delivered2 (rd_kafka_t *rk,
 	       rd_kafka_err2str(rkmessage->err), rkmessage->offset);
         if (rkmessage->err)
 		fprintf(stderr, "%% Message delivery failed: %s\n",
-                        rd_kafka_message_errstr(rkmessage));
+                        rd_kafka_err2str(rkmessage->err));
 	else if (!quiet)
 		fprintf(stderr,
                         "%% Message delivered (%zd bytes, offset %"PRId64", "
@@ -565,8 +565,6 @@ int main (int argc, char **argv) {
 			exit(1);
 		}
 
-		rd_kafka_set_log_level(rk, LOG_DEBUG);
-
 		/* Add brokers */
 		if (rd_kafka_brokers_add(rk, brokers) == 0) {
 			fprintf(stderr, "%% No valid brokers specified\n");
@@ -643,8 +641,6 @@ int main (int argc, char **argv) {
 			exit(1);
 		}
 
-		rd_kafka_set_log_level(rk, LOG_DEBUG);
-
 		/* Add brokers */
 		if (rd_kafka_brokers_add(rk, brokers) == 0) {
 			fprintf(stderr, "%% No valid brokers specified\n");
@@ -747,8 +743,6 @@ int main (int argc, char **argv) {
 			exit(1);
 		}
 
-		rd_kafka_set_log_level(rk, LOG_DEBUG);
-
 		/* Add brokers */
 		if (rd_kafka_brokers_add(rk, brokers) == 0) {
 			fprintf(stderr, "%% No valid brokers specified\n");
