@@ -100,7 +100,7 @@ RdKafka::ErrorCode RdKafka::ProducerImpl::produce (RdKafka::Topic *topic,
                        payload, len,
                        key ? key->c_str() : NULL, key ? key->size() : 0,
                        msg_opaque) == -1)
-    return static_cast<RdKafka::ErrorCode>(rd_kafka_errno2err(errno));
+    return static_cast<RdKafka::ErrorCode>(rd_kafka_last_error());
 
   return RdKafka::ERR_NO_ERROR;
 }
@@ -118,7 +118,7 @@ RdKafka::ErrorCode RdKafka::ProducerImpl::produce (RdKafka::Topic *topic,
   if (rd_kafka_produce(topicimpl->rkt_, partition, msgflags,
                        payload, len, key, key_len,
                        msg_opaque) == -1)
-    return static_cast<RdKafka::ErrorCode>(rd_kafka_errno2err(errno));
+    return static_cast<RdKafka::ErrorCode>(rd_kafka_last_error());
 
   return RdKafka::ERR_NO_ERROR;
 }
@@ -137,8 +137,31 @@ RdKafka::ProducerImpl::produce (RdKafka::Topic *topic,
                        payload ? payload->size() : 0,
                        key ? &(*key)[0] : NULL, key ? key->size() : 0,
                        msg_opaque) == -1)
-    return static_cast<RdKafka::ErrorCode>(rd_kafka_errno2err(errno));
+    return static_cast<RdKafka::ErrorCode>(rd_kafka_last_error());
 
   return RdKafka::ERR_NO_ERROR;
 
 }
+
+
+RdKafka::ErrorCode
+RdKafka::ProducerImpl::produce (const std::string topic_name,
+                                int32_t partition, int msgflags,
+                                void *payload, size_t len,
+                                const void *key, size_t key_len,
+                                int64_t timestamp,
+                                void *msg_opaque) {
+  return
+    static_cast<RdKafka::ErrorCode>
+    (
+     rd_kafka_producev(rk_,
+                       RD_KAFKA_V_TOPIC(topic_name.c_str()),
+                       RD_KAFKA_V_PARTITION(partition),
+                       RD_KAFKA_V_MSGFLAGS(msgflags),
+                       RD_KAFKA_V_VALUE(payload, len),
+                       RD_KAFKA_V_KEY(key, key_len),
+                       RD_KAFKA_V_TIMESTAMP(timestamp),
+                       RD_KAFKA_V_OPAQUE(msg_opaque),
+                       RD_KAFKA_V_END)
+     );
+}
