@@ -41,7 +41,7 @@ struct rd_kafka_broker_s { /* rd_kafka_broker_t */
 #define RD_KAFKA_NODEID_UA -1
 
 	rd_sockaddr_list_t *rkb_rsal;
-	time_t              rkb_t_rsal_last;
+        rd_ts_t             rkb_ts_rsal_last;
         const rd_sockaddr_inx_t  *rkb_addr_last; /* Last used connect address */
 
 	rd_kafka_transport_t *rkb_transport;
@@ -277,10 +277,10 @@ void rd_kafka_broker_fail (rd_kafka_broker_t *rkb,
 
 void rd_kafka_broker_destroy_final (rd_kafka_broker_t *rkb);
 
-static RD_INLINE RD_UNUSED void rd_kafka_broker_destroy (rd_kafka_broker_t *rkb) {
-        rd_refcnt_destroywrapper(&rkb->rkb_refcnt,
-                                 rd_kafka_broker_destroy_final(rkb));
-}
+#define rd_kafka_broker_destroy(rkb)                                    \
+        rd_refcnt_destroywrapper(&(rkb)->rkb_refcnt,                    \
+                                 rd_kafka_broker_destroy_final(rkb))
+
 
 void rd_kafka_broker_update (rd_kafka_t *rk, rd_kafka_secproto_t proto,
                              const struct rd_kafka_metadata_broker *mdb);
