@@ -284,7 +284,7 @@ void rd_kafka_op_destroy (rd_kafka_op_t *rko) {
 		break;
 
         case RD_KAFKA_OP_METADATA:
-                RD_IF_FREE(rko->rko_u.metadata, rd_kafka_metadata_destroy);
+                RD_IF_FREE(rko->rko_u.metadata.md, rd_kafka_metadata_destroy);
                 break;
 
         case RD_KAFKA_OP_LOG:
@@ -453,7 +453,7 @@ rd_kafka_op_t *rd_kafka_op_req (rd_kafka_q_t *destq,
 
         reply = rd_kafka_op_req0(destq, recvq, rko, timeout_ms);
 
-        rd_kafka_q_destroy(recvq);
+        rd_kafka_q_destroy_owner(recvq);
 
         return reply;
 }
