@@ -37,6 +37,7 @@
 #define RD_KAFKA_ERR_ACTION_REFRESH    0x4 /* Refresh state (e.g., metadata) */
 #define RD_KAFKA_ERR_ACTION_RETRY      0x8 /* Retry request after backoff */
 #define RD_KAFKA_ERR_ACTION_INFORM    0x10 /* Inform application about err */
+#define RD_KAFKA_ERR_ACTION_SPECIAL   0x20 /* Special-purpose, depends on context */
 #define RD_KAFKA_ERR_ACTION_END          0 /* var-arg sentinel */
 
 int rd_kafka_err_action (rd_kafka_broker_t *rkb,
@@ -56,13 +57,12 @@ rd_kafka_resp_err_t rd_kafka_handle_Offset (rd_kafka_t *rk,
 					    rd_kafka_resp_err_t err,
 					    rd_kafka_buf_t *rkbuf,
 					    rd_kafka_buf_t *request,
-					    const char *topic, int32_t partition,
-					    int64_t *offsets,
-					    size_t *offset_cntp);
+                                            rd_kafka_topic_partition_list_t
+                                            *offsets);
 
 void rd_kafka_OffsetRequest (rd_kafka_broker_t *rkb,
-                             const char *topic, int32_t partition,
-                             const int64_t *query_offsets, size_t offset_cnt,
+                             rd_kafka_topic_partition_list_t *offsets,
+                             int16_t api_version,
                              rd_kafka_replyq_t replyq,
                              rd_kafka_resp_cb_t *resp_cb,
                              void *opaque);
@@ -105,7 +105,7 @@ int rd_kafka_OffsetCommitRequest (rd_kafka_broker_t *rkb,
 				  rd_kafka_topic_partition_list_t *offsets,
 				  rd_kafka_replyq_t replyq,
 				  rd_kafka_resp_cb_t *resp_cb,
-				  void *opaque);
+				  void *opaque, const char *reason);
 
 
 
@@ -117,12 +117,6 @@ void rd_kafka_JoinGroupRequest (rd_kafka_broker_t *rkb,
                                 rd_kafka_replyq_t replyq,
                                 rd_kafka_resp_cb_t *resp_cb,
                                 void *opaque);
-void rd_kafka_cgrp_handle_JoinGroup (rd_kafka_t *rk,
-				     rd_kafka_broker_t *rkb,
-                                     rd_kafka_resp_err_t err,
-                                     rd_kafka_buf_t *rkbuf,
-                                     rd_kafka_buf_t *request,
-                                     void *opaque);
 
 
 void rd_kafka_LeaveGroupRequest (rd_kafka_broker_t *rkb,
@@ -174,30 +168,11 @@ void rd_kafka_HeartbeatRequest (rd_kafka_broker_t *rkb,
                                 rd_kafka_replyq_t replyq,
                                 rd_kafka_resp_cb_t *resp_cb,
                                 void *opaque);
-void rd_kafka_cgrp_handle_Heartbeat (rd_kafka_t *rk,
-				     rd_kafka_broker_t *rkb,
-                                     rd_kafka_resp_err_t err,
-                                     rd_kafka_buf_t *rkbuf,
-                                     rd_kafka_buf_t *request,
-                                     void *opaque);
 
-rd_kafka_buf_t *rd_kafka_MetadataRequest0 (rd_kafka_broker_t *rkb,
-                                           int all_topics,
-                                           rd_kafka_itopic_t *only_rkt,
-                                           const char *reason);
-void rd_kafka_MetadataRequest (rd_kafka_broker_t *rkb,
-                               int all_topics,
-                               rd_kafka_itopic_t *only_rkt,
-                               const char *reason,
-                               rd_kafka_replyq_t replyq,
-                               rd_kafka_resp_cb_t *resp_cb,
-                               void *opaque);
-void rd_kafka_op_handle_Metadata (rd_kafka_t *rk,
-				  rd_kafka_broker_t *rkb,
-                                  rd_kafka_resp_err_t err,
-                                  rd_kafka_buf_t *rkbuf,
-                                  rd_kafka_buf_t *request,
-                                  void *opaque);
+rd_kafka_resp_err_t
+rd_kafka_MetadataRequest (rd_kafka_broker_t *rkb,
+                          const rd_list_t *topics, const char *reason,
+                          rd_kafka_op_t *rko);
 
 rd_kafka_resp_err_t
 rd_kafka_handle_ApiVersion (rd_kafka_t *rk,
@@ -217,3 +192,5 @@ void rd_kafka_SaslHandshakeRequest (rd_kafka_broker_t *rkb,
 				    rd_kafka_replyq_t replyq,
 				    rd_kafka_resp_cb_t *resp_cb,
 				    void *opaque, int flash_msg);
+
+int rd_kafka_ProduceRequest (rd_kafka_broker_t *rkb, rd_kafka_toppar_t *rktp);
