@@ -31,15 +31,16 @@
 
 
 int rd_kafka_sasl_io_event (rd_kafka_transport_t *rktrans, int events,
-			    char *errstr, int errstr_size);
+			    char *errstr, size_t errstr_size);
+void rd_kafka_sasl_close (rd_kafka_transport_t *rktrans);
 int rd_kafka_sasl_client_new (rd_kafka_transport_t *rktrans,
-			      char *errstr, int errstr_size);
+			      char *errstr, size_t errstr_size);
 
-void rd_kafka_broker_sasl_term (rd_kafka_broker_t *rkb);
-void rd_kafka_broker_sasl_init (rd_kafka_broker_t *rkb);
+void rd_kafka_sasl_broker_term (rd_kafka_broker_t *rkb);
+void rd_kafka_sasl_broker_init (rd_kafka_broker_t *rkb);
 
 void rd_kafka_sasl_global_term (void);
 int rd_kafka_sasl_global_init (void);
 
-int rd_kafka_sasl_conf_validate (rd_kafka_t *rk,
-				 char *errstr, size_t errstr_size);
+int rd_kafka_sasl_select_provider (rd_kafka_t *rk,
+                                   char *errstr, size_t errstr_size);
