@@ -23,6 +23,7 @@ freely, subject to the following restrictions:
 */
 
 #include "rd.h"
+#include "rdtime.h"
 #include "tinycthread.h"
 #include <stdlib.h>
 
@@ -506,6 +507,16 @@ int cnd_timedwait_ms(cnd_t *cnd, mtx_t *mtx, int timeout_ms) {
 #endif
 }
 
+int cnd_timedwait_msp (cnd_t *cnd, mtx_t *mtx, int *timeout_msp) {
+        rd_ts_t pre = rd_clock();
+        int r;
+        r = cnd_timedwait_ms(cnd, mtx, *timeout_msp);
+        if (r != thrd_timedout) {
+                /* Subtract spent time */
+                (*timeout_msp) -= (int)(rd_clock()-pre) / 1000;
+        }
+        return r;
+}
 
 #if defined(_TTHREAD_WIN32_)
 struct TinyCThreadTSSData {
