@@ -34,5 +34,7 @@
 #define WITH_SSL 1
 #define WITH_ZLIB 1
 #define WITH_SNAPPY 1
-
+#define WITH_SASL_SCRAM 1
 #define ENABLE_DEVEL 0
+#define WITH_PLUGINS 1
+#define SOLIB_EXT ".dll"
