@@ -39,8 +39,8 @@
 
 
 static int msgid_next = 0;
-static int msg_remains = 0;
 static int fails = 0;
+static int msgcounter = 0;
 
 /**
  * Delivery reported callback.
@@ -64,6 +64,7 @@ static void dr_single_partition_cb (rd_kafka_t *rk, void *payload, size_t len,
 	}
 
 	msgid_next = msgid+1;
+        msgcounter--;
 }
 
 /* Produce a batch of messages to a single partition. */
@@ -74,7 +75,6 @@ static void test_single_partition (void) {
 	rd_kafka_topic_t *rkt;
 	rd_kafka_conf_t *conf;
 	rd_kafka_topic_conf_t *topic_conf;
-	char errstr[512];
 	char msg[128];
 	int msgcnt = 100000;
 	int failcnt = 0;
@@ -89,10 +89,7 @@ static void test_single_partition (void) {
 	rd_kafka_conf_set_dr_cb(conf, dr_single_partition_cb);
 
 	/* Create kafka instance */
-	rk = rd_kafka_new(RD_KAFKA_PRODUCER, conf,
-			  errstr, sizeof(errstr));
-	if (!rk)
-		TEST_FAIL("Failed to create rdkafka instance: %s\n", errstr);
+	rk = test_create_handle(RD_KAFKA_PRODUCER, conf);
 
 	TEST_SAY("test_single_partition: Created kafka instance %s\n",
 		 rd_kafka_name(rk));
@@ -145,9 +142,10 @@ static void test_single_partition (void) {
 	TEST_SAY("Single partition: "
                  "Produced %i messages, waiting for deliveries\n", r);
 
+        msgcounter = msgcnt;
+
 	/* Wait for messages to be delivered */
-	while (rd_kafka_outq_len(rk) > 0)
-		rd_kafka_poll(rk, 50);
+        test_wait_delivery(rk, &msgcounter);
 
 	if (fails)
 		TEST_FAIL("%i failures, see previous errors", fails);
@@ -182,10 +180,10 @@ static void dr_partitioner_cb (rd_kafka_t *rk, void *payload, size_t len,
 		TEST_FAIL("Message delivery failed: %s\n",
 			  rd_kafka_err2str(err));
 
-        if (msg_remains <= 0)
+        if (msgcounter <= 0)
                 TEST_FAIL("Too many message dr_cb callback calls "
                           "(at msgid #%i)\n", msgid);
-        msg_remains--;
+        msgcounter--;
 }
 
 /* Produce a batch of messages using random (default) partitioner */
@@ -196,25 +194,19 @@ static void test_partitioner (void) {
 	rd_kafka_topic_t *rkt;
 	rd_kafka_conf_t *conf;
 	rd_kafka_topic_conf_t *topic_conf;
-	char errstr[512];
 	char msg[128];
 	int msgcnt = 100000;
         int failcnt = 0;
 	int i;
         rd_kafka_message_t *rkmessages;
 
-        msg_remains = 0;
-
-	test_conf_init(&conf, &topic_conf, 20);
+	test_conf_init(&conf, &topic_conf, 30);
 
 	/* Set delivery report callback */
 	rd_kafka_conf_set_dr_cb(conf, dr_partitioner_cb);
 
 	/* Create kafka instance */
-	rk = rd_kafka_new(RD_KAFKA_PRODUCER, conf,
-			  errstr, sizeof(errstr));
-	if (!rk)
-		TEST_FAIL("Failed to create rdkafka instance: %s\n", errstr);
+	rk = test_create_handle(RD_KAFKA_PRODUCER, conf);
 
 	TEST_SAY("test_partitioner: Created kafka instance %s\n",
 		 rd_kafka_name(rk));
@@ -238,7 +230,6 @@ static void test_partitioner (void) {
                 rkmessages[i]._private = msgidp;
         }
 
-        msg_remains = msgcnt;
         r = rd_kafka_produce_batch(rkt, partition, RD_KAFKA_MSG_F_FREE,
                                    rkmessages, msgcnt);
 
@@ -268,16 +259,16 @@ static void test_partitioner (void) {
 	TEST_SAY("Partitioner: "
                  "Produced %i messages, waiting for deliveries\n", r);
 
+        msgcounter = msgcnt;
 	/* Wait for messages to be delivered */
-	while (rd_kafka_outq_len(rk) > 0)
-		rd_kafka_poll(rk, 50);
+        test_wait_delivery(rk, &msgcounter);
 
 	if (fails)
 		TEST_FAIL("%i failures, see previous errors", fails);
 
-        if (msg_remains != 0)
+        if (msgcounter != 0)
 		TEST_FAIL("Still waiting for %i/%i messages\n",
-                          msg_remains, msgcnt);
+                          msgcounter, msgcnt);
 
 	/* Destroy topic */
 	rd_kafka_topic_destroy(rkt);
