@@ -12,6 +12,7 @@ from trivup.apps.KafkaBrokerApp import KafkaBrokerApp
 from trivup.apps.KerberosKdcApp import KerberosKdcApp
 
 import json
+import subprocess
 
 
 class LibrdkafkaTestApp(App):
@@ -24,13 +25,16 @@ class LibrdkafkaTestApp(App):
         self.appid = UuidAllocator(self.cluster).next(self, trunc=8)
         self.autostart = False
         self.local_tests = True
-        self.test_mode = ''  # Default mode (bare) to run-test.sh
+        self.test_mode = conf.get('test_mode', 'bare')
+        self.version = version
 
         # Generate test config file
         conf_blob = list()
         security_protocol='PLAINTEXT'
 
-        f, test_conf_file = self.open_file('test.conf', 'perm')
+        f, self.test_conf_file = self.open_file('test.conf', 'perm')
+        f.write('broker.address.family=v4\n'.encode('ascii'))
+        f.write(('test.sql.command=sqlite3 rdktests\n').encode('ascii'))
         f.write(('\n'.join(conf_blob)).encode('ascii'))
 
         if version == 'trunk' or version.startswith('0.10.'):
@@ -42,7 +46,7 @@ class LibrdkafkaTestApp(App):
         mech = self.conf.get('sasl_mechanisms', '').split(',')[0]
         if mech != '':
             conf_blob.append('sasl.mechanisms=%s' % mech)
-            if mech == 'PLAIN':
+            if mech == 'PLAIN' or mech.find('SCRAM-') != -1:
                 security_protocol='SASL_PLAINTEXT'
                 # Use first user as SASL user/pass
                 for up in self.conf.get('sasl_users', '').split(','):
@@ -58,14 +62,16 @@ class LibrdkafkaTestApp(App):
                     self.log('WARNING: sasl_mechanisms is GSSAPI set but no KerberosKdcApp available: client SASL config will be invalid (which might be intentional)')
                 else:
                     self.env_add('KRB5_CONFIG', kdc.conf['krb5_conf'])
-                    principal,keytab = kdc.add_principal(self.name, self.node.name)
+                    self.env_add('KRB5_KDC_PROFILE', kdc.conf['kdc_conf'])
+                    principal,keytab = kdc.add_principal(self.name,
+                                                         conf.get('advertised_hostname', self.node.name))
                     conf_blob.append('sasl.kerberos.service.name=%s' % \
-                                     self.conf.get('sasl_servicename'))
+                                     self.conf.get('sasl_servicename', 'kafka'))
                     conf_blob.append('sasl.kerberos.keytab=%s' % keytab)
                     conf_blob.append('sasl.kerberos.principal=%s' % principal.split('@')[0])
 
             else:
-                self.log('WARNING: FIXME: SASL %s client config not written to %s: unhandled mechanism' % (mech, test_conf_file))
+                self.log('WARNING: FIXME: SASL %s client config not written to %s: unhandled mechanism' % (mech, self.test_conf_file))
 
         # SSL config
         if getattr(cluster, 'ssl', None) is not None:
@@ -85,11 +91,11 @@ class LibrdkafkaTestApp(App):
 
         # Define bootstrap brokers based on selected security protocol
         self.dbg('Using client security.protocol=%s' % security_protocol)
-        all_listeners = (','.join(cluster.get_all('listeners', '', KafkaBrokerApp))).split(',')
+        all_listeners = (','.join(cluster.get_all('advertised.listeners', '', KafkaBrokerApp))).split(',')
         bootstrap_servers = ','.join([x for x in all_listeners if x.startswith(security_protocol)])
         if len(bootstrap_servers) == 0:
             bootstrap_servers = all_listeners[0]
-            self.log('WARNING: No eligible listeners for security.protocol=%s: falling back to first listener: %s: tests will fail (which might be the intention)' % (security_protocol, bootstrap_servers))
+            self.log('WARNING: No eligible listeners for security.protocol=%s in %s: falling back to first listener: %s: tests will fail (which might be the intention)' % (security_protocol, all_listeners, bootstrap_servers))
 
         conf_blob.append('bootstrap.servers=%s' % bootstrap_servers)
         conf_blob.append('security.protocol=%s' % security_protocol)
@@ -97,24 +103,27 @@ class LibrdkafkaTestApp(App):
         f.write(('\n'.join(conf_blob)).encode('ascii'))
         f.close()
 
-        self.env_add('RDKAFKA_TEST_CONF', test_conf_file)
+        self.env_add('RDKAFKA_TEST_CONF', self.test_conf_file)
         self.env_add('TEST_KAFKA_VERSION', version)
 
-        self.test_report_file = self.mkpath('test_report', pathtype='perm')
-        self.env_add('TEST_REPORT', self.test_report_file)
+        if self.test_mode != 'bash':
+            self.test_report_file = self.mkpath('test_report', pathtype='perm')
+            self.env_add('TEST_REPORT', self.test_report_file)
 
-        if tests is not None:
-            self.env_add('TESTS', ','.join(tests))
+            if tests is not None:
+                self.env_add('TESTS', ','.join(tests))
 
     def start_cmd (self):
         extra_args = list()
-        mode = self.test_mode
         if not self.local_tests:
             extra_args.append('-L')
-        return './run-test.sh -p5 -K %s ./merged %s' % (' '.join(extra_args), mode)
+        return './run-test.sh -p5 -K %s ./merged %s' % (' '.join(extra_args), self.test_mode)
 
 
     def report (self):
+        if self.test_mode == 'bash':
+            return None
+
         try:
             with open(self.test_report_file, 'r') as f:
                 res = json.load(f)
